#include "stdafx.h"
#include "VirtualCDDevice.h"

using namespace BazisLib;

VirtualCDDevice::VirtualCDDevice( const BazisLib::ConstManagedPointer<ImageFormats::AIParsedCDImage> &pImage )
	: INIT_REFERENCE(m_pImage, pImage)
	, m_SectorCount(0)
	, m_LastReportedDriveLetter(0)
{
	if (m_pImage)
		m_SectorCount = m_pImage->GetSectorCount();
}

ULONGLONG VirtualCDDevice::GetSectorCount()
{
	return m_SectorCount;
}

unsigned VirtualCDDevice::Read( ULONGLONG ByteOffset, void *pBuffer, unsigned Length )
{
	if (!m_pImage)
		return 0;
	unsigned firstSector = (unsigned)(ByteOffset / ImageFormats::CD_SECTOR_SIZE);
	if (firstSector > m_SectorCount)
		return 0;

	unsigned sectorCount = Length / ImageFormats::CD_SECTOR_SIZE;

	if (sectorCount > (m_SectorCount - firstSector))
		sectorCount = m_SectorCount - firstSector;

	return m_pImage->ReadSectorsBoundsChecked(firstSector, pBuffer, sectorCount);
}

unsigned VirtualCDDevice::GetTrackCount()
{
	if (!m_pImage)
		return 0;
	return m_pImage->GetTrackCount();
}

unsigned VirtualCDDevice::GetTrackEndSector( unsigned TrackIndex )
{
	if (!m_pImage)
		return 0;
	return m_pImage->GetTrackEndSector(TrackIndex);
}

bool VirtualCDDevice::DeviceControl( unsigned CtlCode, void *pBuffer, unsigned InSize, unsigned OutSize, unsigned *pBytesDone )
{
	if ((CtlCode & 0xFFFF3FFF) == (IOCTL_MOUNTDEV_LINK_CREATED & 0xFFFF3FFF))
	{
		PMOUNTDEV_NAME pName = (PMOUNTDEV_NAME)pBuffer;
		if (!pName || !pName->NameLength || (pName->NameLength > (InSize + 2)))
			return false;

		String link(pName->Name, pName->NameLength / sizeof(wchar_t));
		if (!link.substr(0, 12).icompare(L"\\DosDevices\\"))
		{
			FastMutexLocker lck(m_LinkNameMutex);
			m_LastReportedLink = link;
			m_LastReportedDriveLetter = m_LastReportedLink[12];
		}
		return true;
	}
	else if ((CtlCode & 0xFFFF3FFF) == (IOCTL_MOUNTDEV_LINK_DELETED & 0xFFFF3FFF))
	{
		FastMutexLocker lck(m_LinkNameMutex);
		m_LastReportedLink.clear();
		m_LastReportedDriveLetter = 0;
		return true;
	}
	return false;
}