#pragma once

#include <bzsddk/scsi/VirtualSCSICdrom.h>
#include <bzscmn/file.h>
#include "../ImageFormats/ImageFormats.h"
#include <bzscmn/sync.h>

class VirtualCDDevice : public BazisLib::DDK::SCSI::VirtualSCSICdrom
{
private:
	DECLARE_REFERENCE(ImageFormats::AIParsedCDImage, m_pImage);
	unsigned m_SectorCount;

	BazisLib::DDK::FastMutex m_LinkNameMutex;
	BazisLib::String m_LastReportedLink;
	char m_LastReportedDriveLetter;

public:
	VirtualCDDevice(const BazisLib::ConstManagedPointer<ImageFormats::AIParsedCDImage> &pImage);

	BazisLib::String GetLastReportedLink()
	{
		BazisLib::FastMutexLocker lck(m_LinkNameMutex);
		return m_LastReportedLink;
	}

	char GetLastReportedDriveLetter()
	{
		return m_LastReportedDriveLetter;
	}

public:
	virtual ULONGLONG GetSectorCount() override;
	virtual unsigned Read(ULONGLONG ByteOffset, void *pBuffer, unsigned Length) override;

	virtual unsigned GetTrackCount() override;
	virtual unsigned GetTrackEndSector(unsigned TrackIndex) override;

	virtual bool DeviceControl(unsigned CtlCode, void *pBuffer, unsigned InSize, unsigned OutSize, unsigned *pBytesDone) override;

	virtual void Dispose() override
	{
		m_pImage = NULL;
	}
};