#include "stdafx.h"
#include "VirtualCDClient.h"
#include <bzscmn/bzsbuf.h>
#include <bzswin/bzsdev.h>
#include "../BazisVirtualCDBus/DeviceControl.h"

#pragma comment(lib, "rpcrt4")
#pragma comment(lib, "newdev")

using namespace BazisLib;
using namespace std;


BazisLib::ActionStatus VirtualCDClient::ConnectDisk( LPCWSTR lpKernelFilePath, char SuggestedLetter)
{
	if (!lpKernelFilePath)
		return MAKE_STATUS(InvalidParameter);
	if (!m_pDevice)
		return MAKE_STATUS(NotInitialized);

	VirtualCDMountParams params = {0,};
	params.DisableAutorun = false;
	params.SuggestedDriveLetter = SuggestedLetter;
	params.StructureVersion = VirtualCDMountParams::CurrentVersion;
	wcsncpy(params.wszFilePath, lpKernelFilePath, __countof(params.wszFilePath));

	ActionStatus status;
	m_pDevice->DeviceIoControl(IOCTL_PLUGIN_DEVICE, &params, (DWORD)sizeof(params), NULL, 0, &status);
	return status;
}

BazisLib::ActionStatus VirtualCDClient::DisconnectDisk( LPCWSTR lpImageFilePath )
{
	if (!lpImageFilePath)
		return MAKE_STATUS(InvalidParameter);
	if (!m_pDevice)
		return MAKE_STATUS(NotInitialized);

	ActionStatus status;
	m_pDevice->DeviceIoControl(IOCTL_PLUGOUT_DEVICE, lpImageFilePath, (DWORD)(wcslen(lpImageFilePath) + 1) * sizeof(wchar_t), NULL, 0, &status);
	return status;
}

VirtualCDClient::VirtualCDClient()
{
	list<String> devs = EnumerateDevicesByInterface(&GUID_BazisVirtualCDBus);
	if (devs.size() < 1)
		return;
	m_pDevice = new OSFile(*devs.begin(), FileFlags::ReadWriteAccess);
}

bool VirtualCDClient::IsDiskConnected( LPCWSTR lpImageFilePath )
{
	if (!lpImageFilePath)
		return false;
	if (!m_pDevice)
		return false;

	ActionStatus status;
	bool present = false;
	m_pDevice->DeviceIoControl(IOCTL_IS_DEVICE_PRESENT, lpImageFilePath, (DWORD)(wcslen(lpImageFilePath) + 1) * sizeof(wchar_t), &present, sizeof(present), &status);
	if (!status.Successful())
		return false;
	return present;
}

bool VirtualCDClient::Win32FileNameToKernelFileName( LPCWSTR lpWin32Name, LPWSTR lpKernelName, size_t MaxKernelNameSize )
{
	if (!lpWin32Name || !lpKernelName || !MaxKernelNameSize)
		return false;
	lpKernelName[0] = 0;
	TCHAR tszFullPath[MAX_STR_SIZE];// = _T("\\??\\");
	GetFullPathName(lpWin32Name, _countof(tszFullPath), tszFullPath, NULL);
	if ((lpWin32Name[0] == '\\') && (lpWin32Name[1] == '\\'))
	{
		wcscpy(lpKernelName, L"\\Device\\LanmanRedirector\\");
		memmove(tszFullPath, tszFullPath + 2, (_tcslen(tszFullPath) - 1) * sizeof(TCHAR));
	}
	else
	{
		TCHAR tszRoot[4] = {tszFullPath[0], ':', '\\', 0};
		/*if (GetDriveType(tszRoot) == DRIVE_REMOTE)
		{
			tszRoot[2] = 0;
			if (!QueryDosDevice(tszRoot, tszFullPath, MAX_PATH))
				return false;
			wcsncpy(lpKernelName, tszFullPath, MAX_PATH);
			GetFullPathName(lpWin32Name, MAX_PATH, tszFullPath, NULL);
			memmove(tszFullPath, tszFullPath + 2, (_tcslen(tszFullPath) - 1) * sizeof(TCHAR));
		}
		else*/
			wcsncpy(lpKernelName, L"\\??\\", MaxKernelNameSize);
	}
	wcsncat(lpKernelName, tszFullPath, MaxKernelNameSize);
	return true;
}

VirtualCDClient::VirtualCDList VirtualCDClient::GetVirtualDiskList(const wchar_t *pFilter)
{
	VirtualCDList devices;

	char buf[8192] = {0,};

	ActionStatus status;
	size_t done = m_pDevice->DeviceIoControl(IOCTL_LIST_DEVICES, pFilter, pFilter ? (wcslen(pFilter) + 1) * sizeof(wchar_t) : 0, &buf, sizeof(buf), &status);
	if (!status.Successful())
		return devices;

	VirtualCDRecordHeader *pHdr = (VirtualCDRecordHeader *)buf;
	if (pHdr->Version != VirtualCDRecordHeader::CurrentVersion)
		return devices;

	if (pHdr->RecordCount > 50)
		pHdr->RecordCount = 50;

	devices.reserve(pHdr->RecordCount);

	for (unsigned i = 0; i < pHdr->RecordCount; i++)
	{
		wchar_t *pImgName = (wchar_t *)(buf + pHdr->Records[i].ImageNameOffset);
		if (!wcsncmp(pImgName, L"\\??\\", 4))
			pImgName += 4;
		else if (!wcsncmp(pImgName, L"\\Device\\LanmanRedirector", 24))
			pImgName += 23, pImgName[0] = '\\';
		devices.push_back(VirtualCDEntry(pImgName, pHdr->Records[i].DriveLetter));
	}

	return devices;
}