// batchmnt.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "../VirtualCDCtl/VirtualCDClient.h"

using namespace BazisLib;

int DisplayHelp()
{
	printf("BATCHMNT.EXE - WinCDEmu batch mounter.\r\n");
	printf("Usage:\r\n");
	printf("batchmnt <image file> [<drive letter>] - mount image file\r\n");
	printf("batchmnt /unmount <image file>         - unmount image file\r\n");
	printf("batchmnt /unmount <drive letter>:      - unmount image file\r\n");
	printf("batchmnt /check   <image file>         - return drive letter as ERORLEVEL\r\n");
	printf("batchmnt /unmountall                   - unmount all images\r\n");
	printf("batchmnt /list                         - list mounted\r\n");
	return 0;
}

int ListVirtualCDs(VirtualCDClient &clt)
{
	VirtualCDClient::VirtualCDList devs = clt.GetVirtualDiskList();
	printf("Listing connected virtual CDs:\n");
	printf("Letter  Image file\n");
	for each(const VirtualCDClient::VirtualCDEntry &entry in devs)
		_tprintf(_T("%c:      %s\n"), entry.DriveLetter, entry.ImageFile.c_str());
	printf("------------------------------\n");
	return 0;
}

int CheckCDImageLetter(VirtualCDClient &clt, const wchar_t *pCDImage, bool Silent = false)
{
	if (!pCDImage)
	{
		printf("BATCHMNT: no CD image specified\n");
		return 0;
	}
	wchar_t wszFullPath[MAX_PATH] = {0,};

	if ((pCDImage[1] == ':') && !pCDImage[2])
		wcsncpy(wszFullPath, pCDImage, _countof(wszFullPath));
	else
		VirtualCDClient::Win32FileNameToKernelFileName(pCDImage, wszFullPath, __countof(wszFullPath));
	
	VirtualCDClient::VirtualCDList devs = clt.GetVirtualDiskList(wszFullPath);
	if (devs.empty())
	{
		if (!Silent)
			_tprintf(_T("%s is not mounted\r\n"), pCDImage);
		return 0;
	}
	else
	{
		if (!Silent)
			_tprintf(_T("%s is mounted to %c:\r\n"), devs[0].ImageFile.c_str(), devs[0].DriveLetter);
		return devs[0].DriveLetter;
	}
}

int DoUnmount(VirtualCDClient &clt, const wchar_t *pCDImage)
{
	if (!pCDImage)
	{
		printf("BATCHMNT: no CD image specified\n");
		return 0;
	}
	wchar_t wszFullPath[MAX_PATH] = {0,};

	if ((pCDImage[1] == ':') && !pCDImage[2])
	{
		VirtualCDClient::VirtualCDList devs = clt.GetVirtualDiskList(pCDImage);
		if (devs.empty())
		{
			_tprintf(_T("%s is not a WinCDEmu drive\n"), pCDImage);
			return 1;
		}
		return DoUnmount(clt, devs[0].ImageFile.c_str());
	}

	VirtualCDClient::Win32FileNameToKernelFileName(pCDImage, wszFullPath, __countof(wszFullPath));
	ActionStatus st = clt.DisconnectDisk(wszFullPath);
	_tprintf(_T("%s\n"), st.GetMostInformativeText().c_str());
	return 0;
}

int DoMount(VirtualCDClient &clt, const wchar_t *pCDImage, const wchar_t *pLetter)
{
	if (!pCDImage)
	{
		printf("BATCHMNT: no CD image specified\n");
		return 0;
	}
	wchar_t wszFullPath[MAX_PATH] = {0,};
	VirtualCDClient::Win32FileNameToKernelFileName(pCDImage, wszFullPath, __countof(wszFullPath));

	ActionStatus st = clt.ConnectDisk(wszFullPath, pLetter ? pLetter[0] : 9);
	_tprintf(_T("%s\n"), st.GetMostInformativeText().c_str());
	return !st.Successful();
}

int _tmain(int argc, _TCHAR* argv[])
{
	if (argc < 2)
		return DisplayHelp();
	if (!_tcsicmp(argv[1], _T("/?")) || !_tcsicmp(argv[1], _T("/help")) || !_tcsicmp(argv[1], _T("-help")) || !_tcsicmp(argv[1], _T("--help")))
		return DisplayHelp();

	VirtualCDClient clt;
	if (!clt.Valid())
	{
		printf("Cannot connect to BazisVirtualCDBus.sys!\n");
		return 1;
	}

	if (!_tcsicmp(argv[1], _T("/list")))
		return ListVirtualCDs(clt);
	else if (!_tcsicmp(argv[1], _T("/unmountall")))
	{
		printf("Unmounting all devices...");
		clt.DisconnectDisk(L"*");
		return 0;
	}
	else if (!_tcsicmp(argv[1], _T("/check")))
		return CheckCDImageLetter(clt, (argc >= 3) ? argv[2] : NULL);
	else if (!_tcsicmp(argv[1], _T("/unmount")))
		return DoUnmount(clt, (argc >= 3) ? argv[2] : NULL);
	else
		return DoMount(clt, (argc >= 2) ? argv[1] : NULL, (argc >= 3) ? argv[2] : NULL);
	return 0;
}

